/*
Copyright (C) 2001 StrmnNrmn

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

#include "stdafx.h"
#include <shlwapi.h>
#include "resource.h"
#include "RomSettingsDialog.h"
#include "DBGConsole.h"
#include "IniFile.h"

static BOOL CALLBACK RomSettingsDialogProc(HWND hWndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam);
static BOOL RomSettingsDialog_OnInitDialog(HWND hWndDlg, WPARAM wParam, LPARAM lParam);
static void RomSettingsDialog_UpdateEntry(HWND hWndDlg);

static LPROMINFO s_pRomInfo = NULL;


void RomSettingsDialog_DoModal(HWND hWndParent, LPROMINFO pRomInfo)
{
	s_pRomInfo = pRomInfo;
	DialogBox(g_hInstance, MAKEINTRESOURCE(IDD_ROMSETTINGS), hWndParent, RomSettingsDialogProc);

}

#define CheckBox_GetCheck(hwnd)			SendMessage(hwnd, BM_GETCHECK, 0,0)
#define CheckBox_SetCheck(hwnd, state)	SendMessage(hwnd, BM_SETCHECK, (WPARAM)state,0)


BOOL CALLBACK RomSettingsDialogProc(HWND hWndDlg, UINT uMsg, WPARAM wParam, LPARAM lParam)
{
	switch (uMsg)
	{
	case WM_INITDIALOG:
		return RomSettingsDialog_OnInitDialog(hWndDlg, wParam, lParam);

	case WM_COMMAND:
		switch (LOWORD(wParam))
		{
		case IDC_EEPROM_CHECK:

			if (CheckBox_GetCheck(GetDlgItem(hWndDlg, IDC_EEPROM_CHECK)) == BST_CHECKED)
			{
				EnableWindow(GetDlgItem(hWndDlg, IDC_SIZE_STATIC), TRUE);
				EnableWindow(GetDlgItem(hWndDlg, IDC_EEPROMSIZE_EDIT), TRUE);
			}
			else
			{
				EnableWindow(GetDlgItem(hWndDlg, IDC_SIZE_STATIC), FALSE);
				EnableWindow(GetDlgItem(hWndDlg, IDC_EEPROMSIZE_EDIT), FALSE);
			}

			return TRUE;

		case IDOK:
			RomSettingsDialog_UpdateEntry(hWndDlg);

			EndDialog(hWndDlg, IDOK);
			return TRUE;
		case IDCANCEL:
			EndDialog(hWndDlg, IDCANCEL);
			return TRUE;
		}
		break;
	case WM_DESTROY:
		// Free any plugins that were found
		return TRUE;

	}


	return FALSE;
}

BOOL RomSettingsDialog_OnInitDialog(HWND hWndDlg, WPARAM wParam, LPARAM lParam)
{

	// Initialise the controls
	SetDlgItemText(hWndDlg, IDC_NAME_EDIT, s_pRomInfo->szGameName);
	SetDlgItemText(hWndDlg, IDC_COMMENT_EDIT, s_pRomInfo->szComment);
	SetDlgItemText(hWndDlg, IDC_COMMENT2_EDIT, s_pRomInfo->szComment2);

	SetDlgItemInt(hWndDlg, IDC_UCODE_EDIT, s_pRomInfo->ucode, FALSE);
	SetDlgItemInt(hWndDlg, IDC_SCALEFACTOR_EDIT, s_pRomInfo->nScaleFactor, FALSE);

	CheckBox_SetCheck(GetDlgItem(hWndDlg, IDC_INCTEXRECT_CHECK), s_pRomInfo->bIncTexRectEdge ? BST_CHECKED : BST_UNCHECKED);
	CheckBox_SetCheck(GetDlgItem(hWndDlg, IDC_DISABLE_TEXCRC_CHECK), s_pRomInfo->bDisableTextureCRC ? BST_CHECKED : BST_UNCHECKED);
	CheckBox_SetCheck(GetDlgItem(hWndDlg, IDC_DISABLE_DYNAREC_CHECK), s_pRomInfo->bDisableDynarec ? BST_CHECKED : BST_UNCHECKED);
	CheckBox_SetCheck(GetDlgItem(hWndDlg, IDC_DISABLE_SPEEDSYNC_CHECK), s_pRomInfo->bDisableSpeedSync ? BST_CHECKED : BST_UNCHECKED);
	CheckBox_SetCheck(GetDlgItem(hWndDlg, IDC_EXPANSIONPAK_CHECK), s_pRomInfo->bExpansionPak ? BST_CHECKED : BST_UNCHECKED);

	// OS HLE

	SetDlgItemInt(hWndDlg, IDC_RESCAN_EDIT, s_pRomInfo->dwRescanCount, FALSE);
	if (s_pRomInfo->bDisablePatches)
	{
		CheckBox_SetCheck(GetDlgItem(hWndDlg, IDC_OSHLE_CHECK), BST_UNCHECKED);
		EnableWindow(GetDlgItem(hWndDlg, IDC_RESCAN1_STATIC), FALSE);
		EnableWindow(GetDlgItem(hWndDlg, IDC_RESCAN2_STATIC), FALSE);
		EnableWindow(GetDlgItem(hWndDlg, IDC_RESCAN_EDIT), FALSE);

	}
	else
	{
		CheckBox_SetCheck(GetDlgItem(hWndDlg, IDC_OSHLE_CHECK), BST_CHECKED);
		EnableWindow(GetDlgItem(hWndDlg, IDC_RESCAN1_STATIC), TRUE);
		EnableWindow(GetDlgItem(hWndDlg, IDC_RESCAN2_STATIC), TRUE);
		EnableWindow(GetDlgItem(hWndDlg, IDC_RESCAN_EDIT), TRUE);

	}



	// Eeprom stuff:
	SetDlgItemInt(hWndDlg, IDC_EEPROMSIZE_EDIT, s_pRomInfo->dwEepromSize, FALSE);
	if (s_pRomInfo->bDisableEeprom)
	{
		CheckBox_SetCheck(GetDlgItem(hWndDlg, IDC_EEPROM_CHECK), BST_UNCHECKED);
		EnableWindow(GetDlgItem(hWndDlg, IDC_SIZE_STATIC), FALSE);
		EnableWindow(GetDlgItem(hWndDlg, IDC_EEPROMSIZE_EDIT), FALSE);
	}
	else
	{
		CheckBox_SetCheck(GetDlgItem(hWndDlg, IDC_EEPROM_CHECK), BST_CHECKED);
		EnableWindow(GetDlgItem(hWndDlg, IDC_SIZE_STATIC), TRUE);
		EnableWindow(GetDlgItem(hWndDlg, IDC_EEPROMSIZE_EDIT), TRUE);
	}



	SetFocus(GetDlgItem(hWndDlg, IDC_NAME_EDIT));

	// We set the focus, return false
	return FALSE;
}


void RomSettingsDialog_UpdateEntry(HWND hWndDlg)
{
	LONG i;
	LONG val;
	BOOL bSuccess;


	// Find out which entry we want to modify
	i = g_pIniFile->FindEntry(s_pRomInfo->rh.dwCRC1,
							  s_pRomInfo->rh.dwCRC2,
							  s_pRomInfo->rh.nCountryID,
							  s_pRomInfo->szGameName);


	// Copy fields across
	GetDlgItemText(hWndDlg, IDC_NAME_EDIT, s_pRomInfo->szGameName, sizeof(s_pRomInfo->szGameName));
	GetDlgItemText(hWndDlg, IDC_COMMENT_EDIT, s_pRomInfo->szComment, sizeof(s_pRomInfo->szComment));
	GetDlgItemText(hWndDlg, IDC_COMMENT2_EDIT, s_pRomInfo->szComment2, sizeof(s_pRomInfo->szComment2));

	val = GetDlgItemInt(hWndDlg, IDC_UCODE_EDIT, &bSuccess, FALSE);
	if (bSuccess)
		s_pRomInfo->ucode = val;
	val = GetDlgItemInt(hWndDlg, IDC_SCALEFACTOR_EDIT, &bSuccess, FALSE);
	if (bSuccess)
		s_pRomInfo->nScaleFactor = val;

	val = GetDlgItemInt(hWndDlg, IDC_EEPROMSIZE_EDIT, &bSuccess, FALSE);
	if (bSuccess)
		s_pRomInfo->dwEepromSize = val;

	val = GetDlgItemInt(hWndDlg, IDC_RESCAN_EDIT, &bSuccess, FALSE);
	if (bSuccess)
		s_pRomInfo->dwRescanCount = val;


	if (CheckBox_GetCheck(GetDlgItem(hWndDlg, IDC_OSHLE_CHECK)) == BST_CHECKED)
		s_pRomInfo->bDisablePatches = FALSE;
	else
		s_pRomInfo->bDisablePatches = TRUE;


	if (CheckBox_GetCheck(GetDlgItem(hWndDlg, IDC_EEPROM_CHECK)) == BST_CHECKED)
		s_pRomInfo->bDisableEeprom = FALSE;
	else
		s_pRomInfo->bDisableEeprom = TRUE;




	s_pRomInfo->bIncTexRectEdge = (CheckBox_GetCheck(GetDlgItem(hWndDlg, IDC_INCTEXRECT_CHECK)) == BST_CHECKED);
	s_pRomInfo->bDisableTextureCRC = (CheckBox_GetCheck(GetDlgItem(hWndDlg, IDC_DISABLE_TEXCRC_CHECK)) == BST_CHECKED);
	s_pRomInfo->bDisableDynarec = (CheckBox_GetCheck(GetDlgItem(hWndDlg, IDC_DISABLE_DYNAREC_CHECK)) == BST_CHECKED);
	s_pRomInfo->bDisableSpeedSync = (CheckBox_GetCheck(GetDlgItem(hWndDlg, IDC_DISABLE_SPEEDSYNC_CHECK)) == BST_CHECKED);
	s_pRomInfo->bExpansionPak = (CheckBox_GetCheck(GetDlgItem(hWndDlg, IDC_EXPANSIONPAK_CHECK)) == BST_CHECKED);

	// Update inifile values
	lstrcpyn(g_pIniFile->sections[i].name, s_pRomInfo->szGameName, sizeof(g_pIniFile->sections[i].name));
	lstrcpyn(g_pIniFile->sections[i].comment, s_pRomInfo->szComment, sizeof(g_pIniFile->sections[i].comment));
	lstrcpyn(g_pIniFile->sections[i].comment2, s_pRomInfo->szComment2, sizeof(g_pIniFile->sections[i].comment2));

	g_pIniFile->sections[i].ucode = s_pRomInfo->ucode;
	g_pIniFile->sections[i].bIncTexRectEdge = s_pRomInfo->bIncTexRectEdge;
	g_pIniFile->sections[i].bDisableTextureCRC = s_pRomInfo->bDisableTextureCRC;
	g_pIniFile->sections[i].bDisableDynarec = s_pRomInfo->bDisableDynarec;
	g_pIniFile->sections[i].bDisablePatches = s_pRomInfo->bDisablePatches;
	g_pIniFile->sections[i].bDisableSpeedSync = s_pRomInfo->bDisableSpeedSync;
	g_pIniFile->sections[i].bDisableEeprom = s_pRomInfo->bDisableEeprom;
	g_pIniFile->sections[i].nScaleFactor = s_pRomInfo->nScaleFactor;
	g_pIniFile->sections[i].bExpansionPak = s_pRomInfo->bExpansionPak;
	g_pIniFile->sections[i].dwEepromSize = s_pRomInfo->dwEepromSize;
	g_pIniFile->sections[i].dwRescanCount = s_pRomInfo->dwRescanCount;

	g_pIniFile->bChanged = TRUE;
}